
! This program generates the random draws used in estimating the random coefficients discrete choice model of limited
! information presented in "Limited Information and Advertising in the US Personal Computer Industry"

! written by Michelle S. Goeree, 2008

program datagenerate

	implicit real*8 (a-h,o-z)
	integer itag
	common/tag/itag
	
	write(6,*)'calling nudraw  subroutine '      
	call nudraw
	write(6,*)'calling kappadraw  subroutine '      
	call kappadraw
	write(6,*)'calling randdraw  subroutine '      
	call randdraw
	write(6,*)'calling simdraw  subroutine '      
	call simdraw

end program datagenerate

	subroutine nudraw
	!	------------ Draw unobserved consumer characteristics, v_i -------------
	!	One draw from MVN for each person for each product characteristic.
	!	ns_t*nnlk iid multivariate normal draws are provided.
	!	----------------------------------------------------------------

	use numerical_libraries
	use linear_operators

	implicit real*8 (a-h, o-z)
	integer, parameter :: ns=9000,nnlk=4,nqrt=12,ns_t=ns/nqrt
	integer i,irank,iseed,j,k,t,vbeg,vend
	real*8 covn(nnlk,nnlk),v_i(ns_t,nnlk),cfcov(nnlk,nnlk)

	covn=eye(nnlk)
	!"square root" of var-cov matrix (the Cholesky factorization)
	call dchfac(nnlk,covn,nnlk,1.d-5,irank,cfcov,nnlk)	

	iseed = 123457				!Initialize seed of random number generator.
	call rnset (iseed)
	call drnmvn(ns_t,nnlk,cfcov,nnlk,v_i,ns_t)		
	
	open(10, file='c:\michelle\pcad\programs\main\nu.txt')
	do i=1,ns_t
		write(10,100)(v_i(i,k),k=1,nnlk)
	enddo
100	format(3(1x,d15.7)) 

	close(10)
		
	return
	end subroutine nudraw

	subroutine kappadraw

	!	------------ generate unobserved consumer ad effectiveness, k_i -------------
	!
	!	One draw from MV log normal for each person for each ad medium
	!	ns_t*nm iid multivariate log normal draws are provided.
	!	----------------------------------------------------------------
	
	use numerical_libraries
	use linear_operators

	implicit real*8 (a-h,o-z)
	integer,parameter :: ns=9000,nm=4,nqrt=12,ns_t=ns/nqrt
	integer i,j,k,t,m,irank,iseed,vbeg,vend
	real*8 covk(nm,nm),k_i(ns_t,nm),nk_i(ns_t,nm),cfcov(nm,nm),minusk_i,mk_i(ns_t,nm)
	covk=eye(nm)
	iseed = 223457				!this is the seed used to initialize random number generator.
	
	call dchfac(nm,covk,nm,1.d-5,irank,cfcov,nm)	
	call rnset (iseed)
	call drnmvn(ns_t,nm,cfcov,nm,nk_i,ns_t)		
	do i=1,ns_t
		do m=1,nm
			k_i(i,m)=dexp(nk_i(i,m))
		enddo
	enddo

	!for antithetic acceleration
	do i=1,ns_t
		do m=1,nm
			minusk_i=-nk_i(i,m)
			mk_i(i,m)=dexp(minusk_i)
		enddo
	enddo
	open(10, file='c:\michelle\pcad\programs\main\kappa.txt')
	open(15, file='c:\michelle\pcad\programs\main\mkappa.txt')
	do i=1,ns_t
		write(10,100)(k_i(i,m),m=1,nm)
		write(15,100)(mk_i(i,m),m=1,nm)
	enddo
100	format(4(1x,d15.7)) 

	close(10)

	return
	end subroutine kappadraw

	subroutine randdraw
	!	------------ Draw random uniform to determine choice set ------
	!	 u_jt: One draw from uniform (0,1) for each (ns,nprod) combination for cps data
	!	----------------------------------------------------------------

	use numerical_libraries
	use linear_operators

	implicit real*8 (a-h, o-z)     
	integer, parameter :: ns=9000,nprod=2112
	integer i,j,r
	real*8 u_jt(ns,nprod)
	
	!for cps draws
	do i=1,ns
		call drnun(nprod,u_jt(i,1:nprod))
	enddo
	open(10, file='c:\michelle\pcad\programs\main\cpsunif.txt')
	do i=1,ns
		write(10,110)(u_jt(i,j),j=1,nprod)				
	enddo
110 format(2112(1x,d15.7))

	close(10)
	close(20)
	close(25)

	return
	end subroutine randdraw



	subroutine simdraw
	!	------------ Draw unobserved consumer characteristics, consv_i and consk_i------
	!
	!	consv_i: Each individual gets one draw from MVN for each loop (R) 
	!			for each product characteristic. 
	!	
	!			ncons_t*nnlk*nr iid multivariate normal draws are provided.		
	!
	!	consk_i: For each individual one draw from MV log normal for each loop (R)
	!			for each ad medium 
	!
	!			ncons_t*nm*nr iid multivariate log normal draws are provided.
	!	----------------------------------------------------------------
		

	use numerical_libraries
	use linear_operators
	implicit real*8 (a-h, o-z)     
	integer, parameter :: nchar=6,ncons=13400,nnlk=4,nadk=1,nr=3,nm=4,ncons_y=6700
	integer i,r,irank,iseed,j,k,t,vbeg,vend
	real*8 covn(nnlk,nnlk),tempv_i(ncons_y,nnlk),cfcov(nnlk,nnlk),consv(ncons_y,nnlk), &
	consv_i(ncons_y,nnlk,nr),covk(nm,nm),ckcov(nm,nm),nk_i(ncons_y,nm),minusk_i,	&
	consk_i(ncons_y,nm,nr),consmk_i(ncons_y,nm,nr)	

	iseed = 323457								 
	call rnset (iseed)
	do r=1,nr

		!computing consv_i
		covn=eye(nnlk)
		!obtain "square root" of var-cov matrix (the Cholesky factorization)
		call dchfac(nnlk,covn,nnlk,1.d-5,irank,cfcov,nnlk)	
		call drnmvn(ncons_y,nnlk,cfcov,nnlk,consv,ncons_y)		
		consv_i(:,:,r)=consv
		!computing consk_i
		covk=eye(nm)
		call dchfac(nm,covk,nm,1.d-5,irank,ckcov,nm)						
		call drnmvn(ncons_y,nm,ckcov,nm,nk_i,ncons_y)		
		do i=1,ncons_y
			do m=1,nm
				consk_i(i,m,r)=dexp(nk_i(i,m))

				!for antithetic acceleration
				minusk_i=-nk_i(i,m)
				consmk_i(i,m,r)=dexp(minusk_i)
			enddo
		enddo

	enddo
			
	open(10, file='c:\michelle\pcad\programs\main\consnu.txt')
	open(15, file='c:\michelle\pcad\programs\main\conskappa.txt')
	open(20, file='c:\michelle\pcad\programs\main\consmkappa.txt')

	do i=1,ncons_y
		do k=1,nnlk
			write(10,100)(consv_i(i,k,r),r=1,nr)
		enddo
		do m=1,nm
			write(15,100)(consk_i(i,m,r),r=1,nr)
			write(20,100)(consmk_i(i,m,r),r=1,nr)
		enddo
	enddo
100	format(3(1x,d15.7)) 

	close(10)
	close(15)
	close(20)

	return
	end subroutine simdraw



